// Autor: Mihai Scortaru
// Copyright (c) 2002 Agora Media SRL
// Acest fisier este distribuit in regim freeware. 
// Nu este permisa modificarea codului sursa fara acordul autorului.
// Acest fisier nu poate fi folosit in scopuri comerciale fara acordul 
//   scris al SC Agora Media SRL.
// Acest fisier este distribuit "asa cum este"; autorul sau Agora Media SRL
//   nu pot fi responsabili pentru eventuale daune provocate direct sau
//   indirect de acest fisier.
// Programul a fost testat utilizand seturile de date folosite la ONI 2002 
//   si a functionat corect in toate aceste situatii.

#include <stdio.h>
#include <stdlib.h>

int N; // numarul nodurilor arborelui
int Nr; // numarul muchiilor adaugate
int nrRezolvate; // numarul nodurilor rezolvate
int nrSol; // numarul muchiilor adaugate
int nodRezolvabil; // nodul rezolvanil curent
int radacina; // radacina arborelui
int muchii1[1000],muchii2[1000]; // extremitatile muchiilor din arbore
int sol1[1000],sol2[1000]; // extremitatile muchiilor adaugate
int grade[1000]; // gradele nodurilor in arbore
int fii[1000]; // sirul fiilor
int noduri[1000]; // nodurile arborelui intr-o parcurgere BF
int parinte[1000]; // sirul parintilor
int primul[1000]; // sir care contine indicii la care incep fiii nodurilor
int ultimul[1000]; // sir care contine indicii la care se termina fiii nodurilor
int utilizat[1000]; // sir care indica daca un nod a fost parcurs
int rezolvat[1000]; // sir care indica daca un nod face parte dintr-un ciclu
int nrFii[1000]; // numarul de fii ai unui nod
int lungimeFir[1000]; // lungimea firului care incepe dintr-un anumit nod
		      // -1 daca in nod nu incepe un fir

void CitesteDate(void){
  FILE *f=fopen("ARBORE.IN","rt");
  fscanf(f,"%d",&N);
  for (int i=0;i<N-1;i++){
    fscanf(f,"%d%d",&muchii1[i],&muchii2[i]);
    muchii1[i]--; // pentru ca varfurile sa aiba indici intre 0 si N-1
    muchii2[i]--;
    grade[muchii1[i]]++;
    grade[muchii2[i]]++;
  }
}

void AlegeRadacina(void){
  for (int i=0;i<N;i++)
    if (grade[i]>1){
      radacina = i;
      return;
    }
}

void ConstruiesteArbore(void){
  noduri[0] = radacina;
  utilizat[radacina] = 1;
  parinte[0] = -1; // radacina nu are parinte
  int nr=1; // numarul nodurilor parcurse
  int nodCurent=0; // indicele nodului curent
  while (nodCurent < N){
    primul[nodCurent] = nr-1; // fiii nodului curent incep in aceasta pozitie
    for (int i=0;i<N-1;i++){
      if (muchii1[i]==noduri[nodCurent] && !utilizat[muchii2[i]]){
	parinte[nr] = nodCurent;
	fii[nr-1] = nr;
	noduri[nr++] = muchii2[i];
	utilizat[muchii2[i]] = 1;
      }
      else if (muchii2[i]==noduri[nodCurent] && !utilizat[muchii1[i]]){
	parinte[nr] = nodCurent;
	fii[nr-1] = nr;
	noduri[nr++] = muchii1[i];
	utilizat[muchii1[i]] = 1;
      }
    }
    ultimul[nodCurent] = nr-2; // fiii nodului curent se termina in aceasta pozitie
    nrFii[nodCurent] = ultimul[nodCurent] - primul[nodCurent] + 1;
    nodCurent++;
  }
}

void UrcaInArbore(int nod){
  // se urca in arbore pe fir pentru a stabili lungimile
  int k = parinte[nod]; // nodul curent din arbore
  int lungimeFirCurent = 1;
  while (nrFii[k] == 1 && k != -1){
    lungimeFir[k] = ++lungimeFirCurent;
    k = parinte[k];
  }
}

void StabilesteFire(){
  for (int i=0;i<N;i++)
    lungimeFir[i] = -1;
  for (i=0;i<N;i++){
    if (!nrFii[i]){
      lungimeFir[i] = 1;
      UrcaInArbore(i);
    }
  }
}

int Rezolvabil(int nod){
  if (lungimeFir[nod] == -1){ // nodul nu face parte din nici un fir
    for (int i=primul[nod];i<=ultimul[nod];i++)
      if (lungimeFir[fii[i]] == -1 && !rezolvat[fii[i]]) // un subarbore fiu nu este fir si nu a fost rezolvat
	return 0; // nodul nu este rezolvabil
  }
  else return 0; // nodul face parte dintr-un fir
  return 1; // nu s-a intalnit nici o conditie pentru care nodul sa nu fie rezolvabil
}

void IdentificaNodRezolvabil(void){
  nodRezolvabil = 0; // in cazul in care a ramas doar un fir se rezolva radacina
  for (int i=0;i<N;i++)
    if (!rezolvat[i])
      if (Rezolvabil(i)){
	nodRezolvabil = i;
	return;
      }
}

int GasesteFiuNerezolvat(int nod){
  for (int i=primul[nod];i<=ultimul[nod];i++)
    if (!rezolvat[fii[i]])
      return fii[i];
  return -1;
}

void RezolvaFir(int nod){
  int nodCurent = nod;
  while (nrFii[nodCurent]){
    nodCurent = GasesteFiuNerezolvat(nodCurent);
    rezolvat[nodCurent] = 1;
    nrRezolvate++;
  }
}

int GasesteFire2(int nod){
  int nrFire2 = 0;
  for (int i=primul[nod]; i<=ultimul[nod];i++)
    if (lungimeFir[fii[i]] <= 2 && !rezolvat[fii[i]])
      nrFire2++;
  return nrFire2;
}

void NuExistaSolutie(void){
  FILE *f=fopen("ARBORE.OUT","wt");
  fputs("-1",f);
  fclose(f);
  exit(0);
}

int GasesteFrunzaFir(int nod){
  int nodCurent = nod;
  while (nrFii[nodCurent]){
    nodCurent = GasesteFiuNerezolvat(nodCurent);
  }
  return nodCurent;
}

void AdaugaMuchie(int extremitate1, int extremitate2){
  sol1[nrSol] = noduri[extremitate1];
  sol2[nrSol++] = noduri[extremitate2];
}

void ConstruiesteCicluri(int nod){
  // se determina cele mai scurte fire, daca exista fire de lungime cel mult 2, atunci acestea vor fi gasite;
  // mai intai se determina lungimile primelor doua fire nerezolvate
  if (lungimeFir[nod]==2 || lungimeFir[nod]==1) // a ramas un singur nod sau doua noduri
    NuExistaSolutie();
  int x = primul[nod];
  while (rezolvat[fii[x]] && x<=ultimul[nod]) // se ajunge la primul fir nerezolvat
    x++;
  int ind1 = fii[x];
  x++;
  while (rezolvat[fii[x]] && x<=ultimul[nod]) // se ajunge la al doilea fir nerezolvat
    x++;
  int ind2 = fii[x];
  if (x>ultimul[nod]) // nu exista doi fii nerezolvati
    ind2 = ind1;
  int min1 = lungimeFir[ind1];
  int min2 = lungimeFir[ind2];
  if (min1 > min2){
    int aux = min1;
    min1 = min2;
    min2 = aux;
    aux = ind1;
    ind1 = ind2;
    ind2 = aux;
  }
  // se cauta in continuare
  for (int i = x + 1; i <= ultimul[nod]; i++)
    if (!rezolvat[fii[i]]){
      if (lungimeFir[fii[i]] <= min1){
	min2 = min1;
	ind2 = ind1;
	ind1 = fii[i];
	min1 = lungimeFir[ind1];
      }
      else if (lungimeFir[fii[i]] < min2){
	ind2 = fii[i];
	min2 = lungimeFir[ind2];
      }
    }
  int extremitate1 = GasesteFrunzaFir(ind1);
  int extremitate2;
  if (ind1 != ind2)
    extremitate2 = GasesteFrunzaFir(ind2);
  else // am ramas doar cu un fir
    extremitate2 = nod;
  AdaugaMuchie(extremitate1, extremitate2);
  // se rezolva restul firelor
  for (i = primul[nod]; i <= ultimul[nod]; i++)
    if (!rezolvat[fii[i]] && fii[i] != ind1 && fii[i] != ind2){
      extremitate1 = fii[i];
      extremitate2 = GasesteFrunzaFir(fii[i]);
      AdaugaMuchie(extremitate1, extremitate2);
    }
}

void RezolvaNod(int nod){
  int nrFire2; // numarul firelor care au lungimea cel mult 2
  nrFire2 = GasesteFire2(nod);
  if (nrFire2>2)
    NuExistaSolutie();
  ConstruiesteCicluri(nod);
  // marcheaza tot subarborele ca fiind rezolvat
  rezolvat[nod] = 1;
  nrRezolvate++;
  for (int i=primul[nod];i<=ultimul[nod];i++)
    if (!rezolvat[fii[i]]){
      rezolvat[fii[i]] = 1;
      nrRezolvate++;
      RezolvaFir(fii[i]);
    }
}

void ActualizeazaFir(int nod){
  // se cauta fiul nerezolvat;
  int fiuNerezolvat = GasesteFiuNerezolvat(nod);
  int lungimeFirCurent;
  if (lungimeFir[fiuNerezolvat] != -1){ // fiul nerezolvat face parte dintr-un fir
    lungimeFirCurent = lungimeFir[nod] = lungimeFir[fiuNerezolvat]+1;
    // se urca in arbore cat timp se urmeaza un singur fir
    int k = parinte[nod];
    while (nrFii[k] == 1 && k != -1){
      lungimeFir[k] = ++lungimeFirCurent;
      k = parinte[k];
    }
  }
}

void ConstruiesteFirNou(int nod){
  lungimeFir[nod] = 1;
  UrcaInArbore(nod);
}

void ActualizeazaArbore(int nod){
  int nodParinte = parinte[nod];
  nrFii[nodParinte]--;
  if (nrFii[nodParinte] == 1) // acum ar putea face parte dintr-un fir;
    ActualizeazaFir(nodParinte);
  else
    if (!nrFii[nodParinte]) // parintele a devenit frunza
      ConstruiesteFirNou(nodParinte);
}

void DeterminaSolutia(void){
  AlegeRadacina();
  ConstruiesteArbore();
  StabilesteFire();
  while (nrRezolvate < N){
    IdentificaNodRezolvabil();
    RezolvaNod(nodRezolvabil);
    ActualizeazaArbore(nodRezolvabil);
  }
}

void ScrieSolutia(void){
  FILE *f=fopen("ARBORE.OUT","wt");
  fprintf(f,"%d\n",nrSol);
  for (int i=0;i<nrSol;i++)
    fprintf(f,"%d %d\n",sol1[i]+1,sol2[i]+1);
  fclose(f);
}

void main(void){
  CitesteDate();
  DeterminaSolutia();
  ScrieSolutia();
}