// Autor: Mihai Scortaru
// Copyright (c) 2002 Agora Media SRL
// Acest fisier este distribuit in regim freeware. 
// Nu este permisa modificarea codului sursa fara acordul autorului.
// Acest fisier nu poate fi folosit in scopuri comerciale fara acordul 
//   scris al SC Agora Media SRL.
// Acest fisier este distribuit "asa cum este"; autorul sau Agora Media SRL
//   nu pot fi responsabili pentru eventuale daune provocate direct sau
//   indirect de acest fisier.
// Programul a fost testat utilizand seturile de date folosite la ONI 2002 
//   si a functionat corect in toate aceste situatii.

#include <stdio.h>
#include <math.h>
#include <values.h>

#define INFINIT 10000
#define EPSILON 0.000001

int huge A[51][51][51]; // tabloul tridimensional
char huge pred[51][51][51]; // tabloul tridimensional al predecesorilor
char huge unghiConvex[51][51][51]; // tablou care indica daca unghiul format din trei turnuri este convex
int turnx[51], turny[51]; // coordonatele turnurilor
int lMax; // lungimes maxima a conturului
int n; // numarul turnurilor
int max; // numarul maxim al turnurilor de pe contur
int turnMax; // ultimul turn al poligonului cu numar maxim de turnuri

void CitesteDate(void){
  FILE *f=fopen("COMITAT.IN","rt");
  fscanf(f,"%d",&n);
  for (int i=1;i<=n;i++)
    fscanf(f,"%d%d",&turnx[i],&turny[i]);
  fscanf(f,"%d",&lMax);
  fclose(f);
}

inline dist(int a, int b){
  // distanta Manhatten dintre turnurile a si b
  return abs(turnx[a]-turnx[b]) + abs(turny[a]-turny[b]);
}

void ConstruiestePoligoaneDegenerate(void){
  // poligoane de forma 0 - i - 0
  for (int i = 1; i<n;i++){
    A[1][0][i] = dist(0,i);
    A[2][i][0] = 2 * dist(0,i);
    pred[2][i][0] = 0;
  }
}

void Initializare(void){
  for (int i=0;i<=n;i++)
    for (int j=0;j<=n;j++)
      for (int k=0;k<=n;k++){
	A[k][i][j] = INFINIT;
	pred[k][i][j] = -1;
      }
}

int Convex(int i, int j, int k){
  if (i==k || j==k)
    return 0;
  if ((long)turnx[i]*(turny[j]-turny[k])+
      (long)turnx[j]*(turny[k]-turny[i])+
      (long)turnx[k]*(turny[i]-turny[j]) < 0)
    return 0;
  double tangenta1 = (turnx[i])?(double)turny[i]/turnx[i]:MAXDOUBLE;
  double tangenta2 = (turnx[j])?(double)turny[j]/turnx[j]:MAXDOUBLE;
  double tangenta3 = (turnx[k])?(double)turny[k]/turnx[k]:MAXDOUBLE;
  int dif12 = (fabs(tangenta2 - tangenta1) > EPSILON);
  int dif23 = (fabs(tangenta3 - tangenta2) > EPSILON);
  if (tangenta2 < tangenta1 && i)
    if (dif12)
      return 0;
  if (tangenta3 < tangenta2)
    if (dif23)
      return 0;
  if (!dif12 && !dif23){
    if (turnx[j] < turnx[i] && turnx[j] < turnx[k]) return 0;
    if (turnx[j] > turnx[i] && turnx[j] > turnx[k]) return 0;
    if (turny[j] < turny[i] && turny[j] < turny[k]) return 0;
    if (turny[j] > turny[i] && turny[j] > turny[k]) return 0;
  }
  return 1;
}

void ConstruiesteLinii(int nrTurnuri){
  // se construiesc liniile frante care incep din Mordor si contin nrTurnuri turnuri
  for (int i=0;i<=n;i++)
    for (int j=0;j<=n;j++)
    if (i!=j){
      int min = INFINIT, predAux = -1;
      int distanta = dist(i,j);
      for (int k=0;k<=n;k++)
	if (unghiConvex[k][i][j]){
	  int d = A[nrTurnuri-1][k][i] + distanta;
	  if (d < min){
	    min = d;
	    predAux = k;
	  }
	}
      A[nrTurnuri][i][j] = min;
      pred[nrTurnuri][i][j] = predAux;
    }
}

void AlegeMaxim(void){
  for (int i=n;i>=0;i--)
    for (int j=1;j<=n;j++)
      if (A[i][j][0] <= lMax){
	max = i;
	turnMax = j;
	return;
      }
}

void DeterminaUnghiuri(void){
  for (int i=0;i<=n;i++)
    for (int j=0;j<=n;j++)
      for (int k=0;k<=n;k++)
	unghiConvex[i][j][k] = Convex(i,j,k);
}

void DeterminaSolutia(void){
  Initializare();
  ConstruiestePoligoaneDegenerate();
  DeterminaUnghiuri();
  for (int i=2; i<=n;i++)
    ConstruiesteLinii(i);
  AlegeMaxim();
}

void ScrieSolutia(void){
  FILE *f=fopen("COMITAT.OUT","wt");
  int turn1 = turnMax, turn2 = 0, aux;
  fprintf(f,"%d\n",max);
  while (turn1){
    fprintf(f,"%d ",turn1);
    aux = pred[max][turn1][turn2];
    turn2 = turn1;
    turn1 = aux;
    max--;
  }
  fclose(f);
}

void main(void){
  CitesteDate();
  DeterminaSolutia();
  ScrieSolutia();
}